<?php

/**
 * Bootstrap file for plugin
 *
 * @category   Shopware
 * @package    Plugins
 * @subpackage Frontend
 * @author     LayoutBuddy Software GmbH
 * @copyright  Copyright © 2013 LayoutBuddy Software GmbH
 */
class Shopware_Plugins_Frontend_LyByLayoutBuddy_Bootstrap extends Shopware_Components_Plugin_Bootstrap
{

    /**
     * Returns info for Plugin Manager
     */
    public function getInfo()
    {
        return array(
            'autor' => 'LayoutBuddy Software GmbH',
            'copyright' => 'Copyright © 2014, LayoutBuddy Software GmbH',
            'version' => $this->getVersion(),
            'label' => $this->getLabel(),
            'supplier' => 'LayoutBuddy Software GmbH',
            'description' => 'LayoutBuddy integration in Shopware',
            'support' => 'info@layoutbuddy.com',
            'link' => 'http://www.layoutbuddy.com',
            'license' => '',
        );
    }

    /**
     * Returns the version
     */
    public function getVersion()
    {
        return "1.1.5";
    }

    /**
     * Returns the label
     */
    public function getLabel()
    {
        return "LayoutBuddy";
    }

    /**
     * Returns capabilities
     */
    /*
    public function getCapabilities()
    {
       return array(
             'install' => true,
             'enable' => true,
             'update' => false
       );
    }
    */

    /**
     * Plugin install method
     */
    public function install()
    {


        $this->subscribeEvents();
        $this->createAttributes();
        $this->createForm();

        $this->updateTo103();
        $this->updateTo104();
        $this->updateTo105();

        return array('success' => true, 'invalidateCache' => array('backend', 'proxy'));
    }

    /**
     * Plugin update method
     */
    public function update($oldVersion)
    {
        switch ($oldVersion) {
            case '1.0.0':
                // nothing to update
            case '1.0.1':
                // nothing to update
            case '1.0.2':
                $this->updateTo103();
            case '1.0.3':
                $this->updateTo104();
            case '1.0.4':
                $this->updateTo105();
            case '1.0.5':
                // nothing to update
                break;
            default:
                return true;
        }
        return true;
    }

    /**
     * Plugin uninstall method
     */
    public function uninstall()
    {
        $this->removeAttributes();
        return true;
    }

    /**
     * Creates config form for keys
     */
    private function createForm()
    {

        $this->Form()->setElement('text', 'lbPublicKey', array(
            'label' => 'Public key',
            'required' => true,
            'scope' => Shopware\Models\Config\Element::SCOPE_SHOP,
        ));

        $this->Form()->setElement('text', 'lbPrivateKey', array(
            'label' => 'Private key',
            'required' => true,
            'scope' => Shopware\Models\Config\Element::SCOPE_SHOP,
        ));
    }

    /**
     * Creates custom attribute fields in the DB
     */
    private function createAttributes()
    {
        if ($this->assertMinimumVersion('5.2')) {

            $this->get('shopware_attribute.crud_service')->update(
                's_articles_attributes',
                'lb_use_for_layout',
                \Shopware\Bundle\AttributeBundle\Service\TypeMapping::TYPE_BOOLEAN,
                [
                    'label' => 'Layout Body Artikel',
                    'helpText' => 'hello world',
                    'displayInBackend' => false,
                    'translatable' => true,
                    'pluginId' => $this->getId()

                ]
            );

            $this->get('shopware_attribute.crud_service')->update(
                's_articles_attributes',
                'lb_config_id',
                \Shopware\Bundle\AttributeBundle\Service\TypeMapping::TYPE_TEXT,
                [
                    'label' => 'Layout Body - Config ID',
                    'helpText' => 'hello world',
                    //'displayInBackend' => true,
                    'displayInBackend' => false,
                    'translatable' => true,
                    'pluginId' => $this->getId()

                ]
            );

            $this->get('shopware_attribute.crud_service')->update(
                's_articles_attributes',
                'lb_layout_id',
                \Shopware\Bundle\AttributeBundle\Service\TypeMapping::TYPE_TEXT,
                [
                    'label' => 'Layout Body - Layout ID',
                    'helpText' => 'hello world',
                    //'displayInBackend' => true,
                    'displayInBackend' => false,
                    'translatable' => true,
                    'pluginId' => $this->getId()
                ]
            );



        } else {
            try {
                //Article
                Shopware()->Models()->addAttribute('s_articles_attributes', 'lb', 'use_for_layout', 'int(1)', true, null);
                Shopware()->Models()->addAttribute('s_articles_attributes', 'lb', 'config_id', 'varchar(255)', true, null);
                Shopware()->Models()->addAttribute('s_articles_attributes', 'lb', 'layout_id', 'varchar(255)', true, null);
            } catch (Exception $e) {

            }
        }

        //Order
        Shopware()->Models()->addAttribute('s_order_basket_attributes', 'lb', 'layout_id', 'varchar(255)', true, null);
        Shopware()->Models()->addAttribute('s_order_basket_attributes', 'lb', 'custom_price', 'double', true, 0);
        Shopware()->Models()->addAttribute('s_order_details_attributes', 'lb', 'layout_id', 'varchar(255)', true, null);

        Shopware()->Models()->getConfiguration()->getMetadataCacheImpl()->deleteAll();
        Shopware()->Models()->generateAttributeModels(array(
            's_articles_attributes',
            's_order_basket_attributes',
            's_order_details_attributes',
        ));
    }

    /**
     * Removes custom attribute fields in the DB
     */
    private function removeAttributes()
    {

        $service = $this->get('shopware_attribute.crud_service');
        //$service->delete('s_articles_attributes', 'lb_use_for_layout');
        //$service->delete('s_articles_attributes', 'lb_config_id');
        //$service->delete('s_articles_attributes', 'lb_layout_id');

        // Shopware()->Models()->removeAttribute('s_articles_attributes', 'lb', 'use_for_layout');
        // Shopware()->Models()->removeAttribute('s_articles_attributes', 'lb', 'config_id');
        // Shopware()->Models()->removeAttribute('s_articles_attributes', 'lb', 'layout_id');

        // Shopware()->Models()->removeAttribute('s_order_basket_attributes', 'lb', 'layout_id');
        // Shopware()->Models()->removeAttribute('s_order_basket_attributes', 'lb', 'custom_price');
        // Shopware()->Models()->removeAttribute('s_order_details_attributes', 'lb', 'layout_id');

        // Shopware()->Models()->getConfiguration()->getMetadataCacheImpl()->deleteAll();
        // Shopware()->Models()->generateAttributeModels(array(
        //         's_articles_attributes',
        //         's_order_basket_attributes',
        //         's_order_details_attributes',
        // ));
    }

    /**
     * Registeres all required events and hooks
     */
    private function subscribeEvents()
    {
        // Backend events

        $this->subscribeEvent(
            'Enlight_Controller_Dispatcher_ControllerPath_Backend_LyByLayoutBuddyOrdersTab',
            'onBackendOrdersTab'
        );

        $this->subscribeEvent(
            'Enlight_Controller_Action_PostDispatch_Backend_Article',
            'onBackendArticlePostDispatch'
        );

        $this->subscribeEvent(
            'Enlight_Controller_Action_PostDispatch_Backend_Order',
            'onBackendOrderPostDispatch'
        );

        $this->subscribeEvent(
            'Shopware_Controllers_Backend_Order::getList::after',
            'onBackendOrderGetListAfter'
        );


        // Frontend events

        $this->subscribeEvent(
            'Enlight_Controller_Action_PostDispatch_Frontend_Detail',
            'onFrontendDetailPostDispatch'
        );

        $this->subscribeEvent(
            'Enlight_Controller_Action_PostDispatch_Frontend_Checkout',
            'onFrontendCheckoutPostDispatch'
        );

        $this->subscribeEvent(
            'Shopware_Modules_Articles_GetArticleById_FilterArticle',
            'onArticle'
        );

        $this->subscribeEvent(
            'Shopware_Modules_Basket_GetBasket_FilterSQL',
            'onGetBasketFilterSql'
        );

        $this->subscribeEvent(
            'Shopware_Modules_Order_SaveOrder_ProcessDetails',
            'onOrderSaveOrderProcessDetails'
        );

        $this->subscribeEvent(
            'sBasket::sUpdateArticle::replace',
            'onUpdateArticle'
        );

    }

    /**
     * Updates to 1.0.3
     */
    protected function updateTo103()
    {
        $this->subscribeEvent(
            'Enlight_Controller_Action_PostDispatch_Frontend_Listing',
            'onFrontendListingPostDispatch'
        );

        $this->subscribeEvent(
            'Shopware_Modules_Articles_sGetArticlesByCategory_FilterResult',
            'onListing'
        );

        $this->subscribeEvent(
            'Shopware_Modules_Order_SendMail_FilterVariables',
            'onOrderSendMailFilterVariables'
        );
    }

    /**
     * Updates to 1.0.4
     */
    protected function updateTo104()
    {
        $this->subscribeEvent(
            'Enlight_Controller_Dispatcher_ControllerPath_Frontend_LyByCheckout',
            'onGetControllerPathFrontendLyByCheckout'
        );
    }

    /**
     * Updates to 1.0.5
     */
    protected function updateTo105()
    {
        $this->Form()->setElement('boolean', 'lbShowOptions', array(
            'label' => 'Optionen zeigen'
        ));

        try {
            Shopware()->Models()->addAttribute('s_order_basket_attributes', 'lb', 'layout_options', 'text', true, null);
            Shopware()->Models()->addAttribute('s_order_details_attributes', 'lb', 'layout_options', 'text', true, null);
        } catch (Exception $e) {

        }
        Shopware()->Models()->getConfiguration()->getMetadataCacheImpl()->deleteAll();
        Shopware()->Models()->generateAttributeModels(array(
            's_order_basket_attributes',
            's_order_details_attributes',
        ));
    }

    /**
     * Helper function to unsubscribe registered events
     */
    protected function unsubscribeEvent($event, $listener, $position = null)
    {
        $namespace = $this->Collection();
        $handler = new Enlight_Event_Handler_Plugin(
            $event, $namespace, $this, $listener, $position
        );
        $namespace->Subscriber()->removeListener($handler);
        return $this;
    }

    /**
     * Helper function to unsubscribe registered hooks
     */
    protected function unsubscribeHook($handler)
    {
        $namespace = $this->Collection();
        $namespace->Subscriber()->removeListener($handler);
        return $this;
    }


    /**
     * Helper method for getting a valid attribute object
     */
    protected function getOrCreateAttribute($object)
    {
        if ($attribute = $object->getAttribute()) {
            return $attribute;
        }

        if ($object instanceof Shopware\Models\Order\Order) {
            if (!$attribute = Shopware()->Models()->getRepository('Shopware\Models\Attribute\Order')
                ->findOneBy(array('orderId' => $object->getId()))
            ) {
                $attribute = new Shopware\Models\Attribute\Order();
            }
        } elseif ($object instanceof Shopware\Models\Order\Detail) {
            if (!$attribute = Shopware()->Models()->getRepository('Shopware\Models\Attribute\OrderDetail')
                ->findOneBy(array('orderDetailId' => $object->getId()))
            ) {
                $attribute = new Shopware\Models\Attribute\OrderDetail();
            }
        } else {
            throw new Exception('Unknown attribute base class');
        }

        $object->setAttribute($attribute);
        return $attribute;
    }

    /**
     * Helper method for reading custom attributes from DB
     */
    protected function getAdditionalArticleAttributes($articleID)
    {
        $result = Shopware()->Db()->fetchRow(
            "SELECT lb_use_for_layout,lb_config_id,lb_layout_id
                  FROM s_articles_attributes 
                  WHERE articleID = ?", array($articleID));

        return $result;
    }


    /**
     * Backend Event for custom controller for LayoutBuddy tab in Backend
     */
    public function onBackendOrdersTab()
    {
        $this->Application()->Snippets()->addConfigDir($this->Path() . 'Snippets/');
        $this->Application()->Template()->addTemplateDir($this->Path() . 'Views/');

        return $this->Path() . 'Controllers/Backend/LyByLayoutBuddyOrdersTab.php';
    }

    /**
     * Backend Event for additional fields in the article's details tab
     */
    public function onBackendArticlePostDispatch(Enlight_Event_EventArgs $args)
    {

        $request = $args->getSubject()->Request();
        $response = $args->getSubject()->Response();
        $view = $args->getSubject()->View();

        $this->Application()->Snippets()->addConfigDir($this->Path() . 'Snippets/');
        $view->addTemplateDir($this->Path() . 'Views/');

        if ($args->getRequest()->getActionName() === 'load') {
            $view->extendsTemplate('backend/layoutbuddy/article/controller/detail.js');
            $view->extendsTemplate('backend/layoutbuddy/article/model/article.js');
            $view->extendsTemplate('backend/layoutbuddy/article/view/detail/base.js');
        }
    }

    /**
     * Backend Event for additional columns in the order's position tab
     */
    public function onBackendOrderPostDispatch(Enlight_Event_EventArgs $args)
    {

        $request = $args->getSubject()->Request();
        $response = $args->getSubject()->Response();
        $view = $args->getSubject()->View();

        $this->Application()->Snippets()->addConfigDir($this->Path() . 'Snippets/');
        $view->addTemplateDir($this->Path() . 'Views/');

        if ($args->getRequest()->getActionName() === 'load') {
            $view->extendsTemplate('backend/layoutbuddy/order/view/detail/tab.js');
            $view->extendsTemplate('backend/layoutbuddy/order/model/position.js');
            $view->extendsTemplate('backend/layoutbuddy/order/view/detail/position.js');
        }
    }

    /**
     * Backend Event for adding custom attributes to order list
     */
    public function onBackendOrderGetListAfter(Enlight_Event_EventArgs $args)
    {
        $return = $args->getReturn();
        if (empty($return['success']) || empty($return['data'])) return;

        foreach ($return['data'] as &$order) {
            foreach ($order["details"] as &$orderDetail) {
                $detailObj = Shopware()->Models()->getRepository('Shopware\Models\Order\Detail')->find($orderDetail['id']);
                $attribute = $this->getOrCreateAttribute($detailObj);
                $orderDetail['lbLayoutId'] = $attribute->getLbLayoutId();
            }
        }

        $args->setReturn($return);
    }


    /**
     * Frontend Event for custom checkout controller
     */
    public function onGetControllerPathFrontendLyByCheckout()
    {
        $this->Application()->Snippets()->addConfigDir($this->Path() . 'Snippets/');
        $this->Application()->Template()->addTemplateDir($this->Path() . 'Views/');

        return $this->Path() . 'Controllers/Frontend/LyByCheckout.php';
    }

    /**
     * Frontend Event for showing the listing page, disable buynow buttons for LayoutBuddy articles
     */
    public function onFrontendListingPostDispatch(Enlight_Event_EventArgs $args)
    {

        $request = $args->getSubject()->Request();
        $response = $args->getSubject()->Response();
        $view = $args->getSubject()->View();

        if (!$request->isDispatched()
            || $response->isException()
            || $request->getModuleName() != 'frontend'
            || !$args->getSubject()->View()->hasTemplate()
        ) {
            return;
        }

        $view->addTemplateDir($this->Path() . 'Views/');
        $view->extendsTemplate('frontend/plugins/layoutbuddy/box_article.tpl');

    }

    /**
     * Frontend Event for showing the detail page of an article, if enabled for the article with the LayoutBuddy tool
     */
    public function onFrontendDetailPostDispatch(Enlight_Event_EventArgs $args)
    {

        $request = $args->getSubject()->Request();
        $response = $args->getSubject()->Response();
        $view = $args->getSubject()->View();

        if (!$request->isDispatched()
            || $response->isException()
            || $request->getModuleName() != 'frontend'
            || !$args->getSubject()->View()->hasTemplate()
        ) {
            return;
        }

        $config = $this->Config();

        if (empty($config->lbPrivateKey) || !$config->lbPrivateKey) return;
        if (empty($config->lbPublicKey) || !$config->lbPublicKey) return;

        $article = $view->sArticle;

        if (empty($article['lb_use_for_layout']) || !((int)$article['lb_use_for_layout'])) return;
        if (empty($article['lb_config_id']) || ($article['lb_config_id']) == '') return;

        $view->layoutbuddyUseForLayout = true;
        $view->layoutbuddyConfigId = $article['lb_config_id'];
        $view->layoutbuddyLayoutId = $article['lb_layout_id'];

        $view->layoutbuddyPrivateKey = $config->lbPrivateKey;
        $view->layoutbuddyPublicKey = $config->lbPublicKey;
        $view->layoutbuddyLocale = Shopware()->Locale()->toString();

        $view->layoutbuddyEmbedUrl = Shopware_Plugins_Frontend_LyByLayoutBuddy_Classes_Api::JS_EMBED_URL;

        $view->addTemplateDir($this->Path() . 'Views/');
        $view->extendsTemplate('frontend/plugins/layoutbuddy/blocks_detail.tpl');

    }

    /**
     * Frontend Event for showing custom images in the cart
     */
    public function onFrontendCheckoutPostDispatch(Enlight_Event_EventArgs $args)
    {

        $request = $args->getSubject()->Request();
        $response = $args->getSubject()->Response();
        $view = $args->getSubject()->View();

        if (!$request->isDispatched()
            || $response->isException()
            || $request->getModuleName() != 'frontend'
            || !$args->getSubject()->View()->hasTemplate()
        ) {
            return;
        }

        if (($config = $this->Config()) && !empty($config->lbShowOptions) && $config->lbShowOptions) {
            $view->layoutbuddyShowOptions = true;
        }

        $layoutbuddyPreviewInfo = array();

        foreach ($view->sBasket['content'] as $item) {
            if (isset($item['lb_layout_id']) && $item['lb_layout_id']) {
                $layoutbuddyPreviewInfo[$item['id']] = array(
                    'thumb' => Shopware_Plugins_Frontend_LyByLayoutBuddy_Classes_Api::getThumbnailImage($item['lb_layout_id']),
                    'preview' => Shopware_Plugins_Frontend_LyByLayoutBuddy_Classes_Api::getPreviewImage($item['lb_layout_id']),
                );
            }
        }
        $view->layoutbuddyPreviewInfo = $layoutbuddyPreviewInfo;

        $view->addTemplateDir($this->Path() . 'Views/');
        $view->extendsTemplate('frontend/plugins/layoutbuddy/blocks_checkout.tpl');

    }


    /**
     * Frontend Event for adding custom attribtues to listing's articles
     */
    public function onListing(Enlight_Event_EventArgs $args)
    {
        $category = $args->getReturn();
        foreach ($category['sArticles'] as &$article) {
            $attributes = $this->getAdditionalArticleAttributes($article['articleID']);
            if ($attributes) {
                foreach ($attributes as $key => $value) {
                    $article[$key] = $value;
                }
            }
        }
        $args->setReturn($category);
    }

    /**
     * Frontend Event for adding custom attribtues to article
     */
    public function onArticle(Enlight_Event_EventArgs $args)
    {
        $article = $args->getReturn();
        $attributes = $this->getAdditionalArticleAttributes($article['articleID']);
        if ($attributes) {
            foreach ($attributes as $key => $value) {
                $article[$key] = $value;
            }
        }
        return $article;
    }

    /**
     * Frontend Event to select custom attributes from DB
     */
    public function onGetBasketFilterSql(Enlight_Event_EventArgs $args)
    {
        $sql = $args->getReturn();
        $sql = str_replace('FROM s_order_basket', ', s_order_basket_attributes.lb_layout_id, s_order_basket_attributes.lb_layout_options FROM s_order_basket', $sql);
        return $sql;
    }

    /**
     * Frontend Event to copy custom attributes (layoutId) from basket to order
     */
    public function onOrderSaveOrderProcessDetails(Enlight_Event_EventArgs $args)
    {
        $details = $args->getDetails();
        foreach ($details as $key => $detail) {
            if (isset($detail['lb_layout_id'])) {
                $result = Shopware()->Db()->update(
                    's_order_details_attributes',
                    array(
                        'lb_layout_id' => $detail['lb_layout_id'],
                        'lb_layout_options' => $detail['lb_layout_options'],
                    ),
                    array('detailID = ?' => $detail['orderDetailId'])
                );
            }
        }
    }

    /**
     * Frontend Event to add custom attribtues to mails
     */
    public function onOrderSendMailFilterVariables(Enlight_Event_EventArgs $args)
    {
        $orderData = $args->getReturn();

        foreach ($orderData['sOrderDetails'] as &$orderDetail) {
            $orderDetailId = $orderDetail['orderDetailId'];
            $detailObj = Shopware()->Models()->getRepository('Shopware\Models\Order\Detail')->find($orderDetailId);
            $attribute = $this->getOrCreateAttribute($detailObj);

            $orderDetail['lbLayoutId'] = $attribute->getLbLayoutId();
            $orderDetail['lbLayoutThumbnail'] = Shopware_Plugins_Frontend_LyByLayoutBuddy_Classes_Api::getThumbnailImage($attribute->getLbLayoutId());
            if (($config = $this->Config()) && !empty($config->lbShowOptions) && $config->lbShowOptions) {
                $orderDetail['lbLayoutOptions'] = $attribute->getLbLayoutOptions();
            }
        }

        $args->setReturn($orderData);
    }

    /**
     * Frontend Event to prevent resetting an article's price to it's original price (for LayoutBuddy products)
     */
    static function onUpdateArticle(Enlight_Hook_HookArgs $args)
    {
        $arguments = $args->getArgs();
        $id = intval($arguments[0]);
        $customPrice = null;
        $layoutId = null;

        if ($id) {
            $result = Shopware()->Db()->fetchRow(
                "SELECT lb_custom_price,lb_layout_id
                  FROM s_order_basket_attributes 
                  WHERE basketID = ?", array($id));

            if ($result['lb_custom_price'] && $result['lb_layout_id']) {
                $customPrice = $result['lb_custom_price'];
                $layoutId = $result['lb_layout_id'];
            }
        }

        Shopware_Plugins_Frontend_LyByLayoutBuddy_Classes_sBasket::sUpdateArticle(
            $args->getSubject(),
            $arguments[0],
            $arguments[1],
            $customPrice,
            $layoutId
        );
    }


    /**
     * Frontend Event to prevent increasing the quantitiy of a single product in the cart, but add as a new one (for LayoutBuddy products)
     */
    static function onAddArticle(Enlight_Hook_HookArgs $args)
    {

    }

    /**
     * Frontend Event for checkout controller to add custom attribute (layoutId) to basket
     */
    public function onAddArticleAction(Enlight_Event_EventArgs $args)
    {
        return true;
    }

}