<?php

/**
 * LayoutBuddy Api Gateway
 *
 * @category   Shopware
 * @package    Plugins
 * @subpackage Frontend
 * @author     LayoutBuddy Software GmbH
 * @copyright  Copyright © 2013 LayoutBuddy Software GmbH
 */

class Shopware_Plugins_Frontend_LyByLayoutBuddy_Classes_Api
{
    
    const VERSION = 0.3;
    
    const API_URL = 'https://www.layoutbuddy.com/api/';
    const STATIC_URL = 'https://s3-eu-west-1.amazonaws.com/static.layoutbuddy.com/';
    const JS_EMBED_URL = '//tool.layoutbuddy.com/embed/';
    
    
    private $_publicKey;
    private $_privateKey;
    
    
    /**
     * Helper method to get the right path for images
     */
    protected static function getSubfolderPath($filename)
    {
        return $filename{0} . '/' . $filename{1} . '/' . $filename;
    }

    /**
     * Returns thumbnail path (120px)
     */
    public static function getThumbnailImage($layoutId)
    {
        return self::STATIC_URL . '120/' . self::getSubfolderPath($layoutId) . '.jpg';
    }

    /**
     * Returns preview path (600px)
     */
    public static function getPreviewImage($layoutId)
    {
        return self::STATIC_URL . '600/' . self::getSubfolderPath($layoutId) . '.jpg';
    }
    
    
    
    /**
     * Constructor, needs public and private key
     */
    public function __construct($publicKey, $privateKey)
    {
        if (!$publicKey || !$privateKey) {
            throw new Exception('Public and private keys must be set.');
        }
        
        $this->_publicKey = $publicKey;
        $this->_privateKey = $privateKey;
    }
    
    /**
     * Returns a secure signed Url
     */
    public function buildSignedRequestUrl($method, $params)
    {
        // Prepare parameters
        $params['public_key'] = $this->_publicKey;
        $params['timestamp'] = time();
        if (isset($params['signature'])) unset($params['signature']);
        
        // Generate signature
        ksort($params);
        $message = $method . '?' . http_build_query($params, '', '&');
        $signature = hash_hmac('sha1', $message, $this->_privateKey, false);
        
        // Add signature
        $params['signature'] = $signature;
        
        $signedUrl = self::API_URL . $method . '?' . http_build_query($params, '', '&');
        return $signedUrl;
    }
    
    /**
     * Returns result of API call
     */
    public function request($method, $params = array(), $auth = false)
    {
        // If authentication required, prepare signed request
        if ($auth) {
            $url = $this->buildSignedRequestUrl($method, $params);
        } else {
            $params['public_key'] = $this->_publicKey;
            $url = self::API_URL . $method . '?' . http_build_query($params, '', '&');
        }

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        return curl_exec($ch);
    }
    
    /**
     * Returns json decoded result of API call
     */
    public function jsonRequest($method, $params = array(), $auth = false)
    {
        return json_decode($this->request($method, $params, $auth));
    }
    
}
